package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.Account;
import com.jair.entity.bo.Company;
import com.jair.entity.bo.Menu;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.constant.AccountTypeEnum;
import com.jair.entity.po.*;
import com.jair.mapper.AccountMapper;
import com.jair.service.IAccountService;
import com.jair.service.ICompanyService;
import com.jair.service.IMenuService;
import com.jair.util.BeanUtil;
import com.jair.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.security.ConcurrentMessageDigest;
import org.apache.tomcat.util.security.MD5Encoder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 帐号 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
@Slf4j
public class AccountServiceImpl extends ServiceImpl<AccountMapper, Account> implements IAccountService {

    @Autowired
    private ICompanyService companyService;
    @Autowired
    private IMenuService menuService;

    @Override
    public ResultVo login(HttpServletRequest request, HttpServletResponse response, LoginPo loginPo) {
        if (loginPo == null) {
            return ResultVo.businessError("请输入用户名、密码");
        }
        if (StringUtils.isEmpty(loginPo.getUsername())) {
            return ResultVo.businessError("请输入用户名");
        }
        if (StringUtils.isEmpty(loginPo.getPassword())) {
            return ResultVo.businessError("请输入密码");
        }

        QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Account::getUsername, loginPo.getUsername());
        Account account = this.getOne(queryWrapper);
        if (account == null) {
            return ResultVo.businessError("用户名不存在");
        }
        if (!account.getPassword().equalsIgnoreCase(loginPo.getPassword())) {
            return ResultVo.businessError("密码不正确");
        }
        if (account.getStatus() == 0) {
            return ResultVo.businessError("该帐号已停用");
        }
        if (account.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
            return loginSuccess(request, response, account, null);
        }

        Company company = companyService.getById(account.getCompanyId());
        if (company == null) {
            return ResultVo.businessError("未找到该帐号关联的企业");
        }
        if (company.getStatus() == 0) {
            return ResultVo.businessError("该企业已停用");
        }

        return loginSuccess(request, response, account, company);
    }

    @Override
    public ResultVo loginSuccess(HttpServletRequest request, HttpServletResponse response, Account account, Company company) {
        Date expiredDate = DateUtil.getAddDayTime(7);
        String token = generateToken(account, expiredDate);

        account.setToken(token);
        account.setTokenExpiredTime(expiredDate);
        this.updateById(account);

        TokenInfo tokenInfo = BeanUtil.transform(account, TokenInfo.class);
        if (company != null) {
            tokenInfo.setCompanyId(company.getId());
            tokenInfo.setCompanyName(company.getName());
            tokenInfo.setAbbr(company.getAbbr());
        }
        tokenInfo.setMenus(this.getAccountMenus(account.getType()));
        response.addHeader("Token", token);

        return ResultVo.success(tokenInfo);
    }

    private String getAccountMenus(Integer accountType) {
        QueryWrapper<Menu> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Menu::getAccountType, accountType);
        Menu menu = menuService.getOne(queryWrapper);
        if (menu == null) {
            return null;
        }
        return menu.getMenus();
    }

    @Override
    public String generateToken(Account account, Date date) {
        String key = account.getUsername() + DateUtil.format2Datetime(date);
        return MD5Encoder.encode(ConcurrentMessageDigest.digestMD5(key.getBytes()));
    }

    @Override
    public TokenInfo getTokenInfo(HttpServletRequest request) {
        String authHeader = request.getHeader("Token");
        //请求头不带Token
        if (StringUtils.isEmpty(authHeader) || !authHeader.startsWith("Bearer ")) {
            log.error("header is empty");
            return null;
        }

        String token = authHeader.substring("Bearer ".length());
        if (StringUtils.isEmpty(token)) {
            log.error("token is empty");
            return null;
        }

        QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Account::getToken, token);
        Account account = this.getOne(queryWrapper);
        if (account == null) {
            log.error("not found account by token,{}", token);
            return null;
        }
        if (account.getTokenExpiredTime().getTime() < new Date().getTime()) {
            log.error("token expired time invalid,{}", token);
            return null;
        }
        if (account.getStatus() == 0) {
            log.error("account is disabled,{}", token);
            return null;
        }
        TokenInfo tokenInfo = BeanUtil.transform(account, TokenInfo.class);
//        Company company = companyMapper.selectById(account.getCompanyId());
//        if (company != null) {
//            tokenInfo.setCompanyId(company.getId());
//            tokenInfo.setCompanyName(company.getName());
//        }

        return tokenInfo;
    }

    @Override
    public List<Account> listAccounts(TokenInfo tokenInfo, Integer companyId, Integer accountType) {
        QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Account::getCompanyId, companyId);
        if (accountType != null)
            queryWrapper.lambda().eq(Account::getType, accountType);
        queryWrapper.orderByAsc("type", "id");
        return this.list(queryWrapper);
    }

    @Override
    public Account getMainAccount(TokenInfo tokenInfo, Integer companyId) {
        List<Account> accounts = this.listAccounts(tokenInfo, companyId, AccountTypeEnum.MAIN.getKey());
        if (accounts == null || accounts.isEmpty()) {
            return null;
        }
        return accounts.get(0);
    }

    @Override
    public ResultVo createAccount(TokenInfo tokenInfo, CreateAccountPo createAccountPo) {
        ValidateResult validate = createAccountPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }

        // 非管理员，并且添加的帐号不是本企业
        if (!tokenInfo.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
            if (!tokenInfo.getCompanyId().equals(createAccountPo.getCompanyId())) {
                return ResultVo.businessError("非管理员，并且添加的帐号不是本企业");
            }
        }

        Company company = this.companyService.getById(createAccountPo.getCompanyId());
        if (company == null) {
            return ResultVo.businessError("未知的企业");
        }

        if (createAccountPo.getType().equals(AccountTypeEnum.MAIN.getKey())) {
            // 非管理员不允许添加主帐号
            if (!tokenInfo.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
                return ResultVo.businessError("非管理员不允许添加主帐号");
            }

            // 每个企业只能存在一个主帐号
            List<Account> mainAccounts = listAccounts(tokenInfo, company.getId(), AccountTypeEnum.MAIN.getKey());
            if (mainAccounts != null && mainAccounts.size() > 0) {
                return ResultVo.businessError("每个企业只能存在一个主帐号");
            }
        } else {
            List<Account> subAccounts = this.listAccounts(tokenInfo, company.getId(), AccountTypeEnum.SUB.getKey());
            int accountSize = 0;
            if (company.getLevel() == 1) {
                accountSize = 5;
            } else if (company.getLevel() == 2) {
                accountSize = 10;
            } else if (company.getLevel() == 3) {
                accountSize = 50;
            }
            if (subAccounts != null && subAccounts.size() >= accountSize) {
                return ResultVo.businessError("企业子帐号数量超过最大限度" + accountSize);
            }
        }

        String accountName = createAccountPo.getUsername() + "@" + company.getAbbr();
        if (this.isUsernameExists(accountName)) {
            return ResultVo.businessError("帐号已经存在");
        }
        if (this.isPhoneExists(null, createAccountPo.getPhone())) {
            return ResultVo.businessError("电话号码已经存在");
        }

        String password = MD5Encoder.encode(ConcurrentMessageDigest.digestMD5(createAccountPo.getPassword().getBytes()));

        Account account = BeanUtil.transform(createAccountPo, Account.class);
        account.setUsername(accountName);
        account.setPassword(password);
        account.setCreateTime(new Date());
        account.setCreateById(tokenInfo.getId());
        account.setCreateByName(tokenInfo.getRealName());
        account.setCompanyId(company.getId());
        account.setCompanyName(company.getName());
        account.setStatus(1);
        this.save(account);

        return ResultVo.success(account);
    }

    @Override
    public ResultVo updateAccount(TokenInfo tokenInfo, UpdateAccountPo updateAccountPo) {
        ValidateResult validate = updateAccountPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }

        Account account = this.getAccountById(tokenInfo.getCompanyId(), updateAccountPo.getId());
        if (account == null) {
            return ResultVo.businessError("未知帐号");
        }

        if (this.isPhoneExists(updateAccountPo.getId(), updateAccountPo.getPhone())) {
            return ResultVo.businessError("电话号码已经存在");
        }

        account.setRealName(updateAccountPo.getRealName());
        account.setPhone(updateAccountPo.getPhone());
        account.setAddress(updateAccountPo.getAddress());
        this.updateById(account);

        return ResultVo.success(account);
    }

    @Override
    public ResultVo updatePassword(TokenInfo tokenInfo, UpdatePasswordPo updatePasswordPo) {
        ValidateResult validate = updatePasswordPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }

        Account account = null;
        if (tokenInfo.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
            // 管理员允许修改所有人密码
            account = this.getAccountById(updatePasswordPo.getId());
        } else {
            account = this.getAccountById(updatePasswordPo.getId(), tokenInfo.getCompanyId());

            // 非企业主帐号仅能修改自己的密码
            if (!tokenInfo.getType().equals(AccountTypeEnum.MAIN.getKey()) && !tokenInfo.getId().equals(updatePasswordPo.getId())) {
                return ResultVo.businessError("非企业主帐号仅能修改自己的密码");
            }
        }
        if (account == null) {
            return ResultVo.businessError("未知帐号");
        }

        String password = MD5Encoder.encode(ConcurrentMessageDigest.digestMD5(updatePasswordPo.getPassword().getBytes()));
        account.setPassword(password);
        this.updateById(account);

        return ResultVo.success(account);
    }

    @Override
    public ResultVo updateStatus(TokenInfo tokenInfo, UpdateAccountStatusPo updateStatusPo) {
        ValidateResult validate = updateStatusPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }
        Account account = null;
        if (tokenInfo.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
            // 管理员允许修改所有人密码
            account = this.getAccountById(updateStatusPo.getId());
        } else {
            // 非企业主帐号仅能修改自己的密码
            if (!tokenInfo.getType().equals(AccountTypeEnum.MAIN.getKey())) {
                return ResultVo.businessError("非企业主帐号不能修改状态");
            }
            account = this.getAccountById(updateStatusPo.getId(), tokenInfo.getCompanyId());
        }
        if (account == null) {
            return ResultVo.businessError("未知帐号");
        }
        account.setStatus(updateStatusPo.getStatus());
        this.updateById(account);

        return ResultVo.success();
    }

    @Override
    public Account getAccountById(Integer accountId, Integer companyId) {
        QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Account::getCompanyId, companyId);
        queryWrapper.lambda().eq(Account::getId, accountId);
        return this.getOne(queryWrapper);
    }

    @Override
    public Account getAccountById(Integer accountId) {
        QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Account::getId, accountId);
        return this.getOne(queryWrapper);
    }

    @Override
    public boolean isUsernameExists(String username) {
        QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Account::getUsername, username);
        return this.getOne(queryWrapper) != null;
    }

    @Override
    public boolean isPhoneExists(Integer accountId, String phone) {
        QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Account::getPhone, phone);
        if (accountId != null)
            queryWrapper.lambda().ne(Account::getId, accountId);
        return this.getOne(queryWrapper) != null;
    }
}
