package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.Account;
import com.jair.entity.bo.Company;
import com.jair.entity.common.PageInfo;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.constant.AccountTypeEnum;
import com.jair.entity.po.CompanyPo;
import com.jair.entity.po.SearchCompanyPo;
import com.jair.entity.po.ValidateResult;
import com.jair.entity.vo.CompanyVo;
import com.jair.mapper.CompanyMapper;
import com.jair.service.IAccountService;
import com.jair.service.ICompanyService;
import com.jair.util.BeanUtil;
import com.jair.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 企业主体 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
@Slf4j
public class CompanyServiceImpl extends ServiceImpl<CompanyMapper, Company> implements ICompanyService {

    @Autowired
    private CompanyMapper companyMapper;
    @Autowired
    private IAccountService accountService;

    @Override
    public ResultVo createCompany(TokenInfo tokenInfo, CompanyPo companyPo) {
        if (!tokenInfo.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
            return ResultVo.businessError("帐号权限不足");
        }
        ValidateResult validate = companyPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }
        if (isCompanyNameExists(null, companyPo.getName())) {
            return ResultVo.businessError("企业名称已经存在");
        }
        if (isCompanyAbbrExists(null, companyPo.getAbbr())) {
            return ResultVo.businessError("企业简称已经存在");
        }

        Company company = BeanUtil.transform(companyPo, Company.class);
        company.setCreateTime(new Date());
        company.setCreateById(tokenInfo.getId());
        company.setCreateByName(tokenInfo.getRealName());
        company.setStatus(1);

        companyMapper.insert(company);
        return ResultVo.success();
    }

    @Override
    public ResultVo updateExpiredDate(TokenInfo tokenInfo, CompanyPo companyPo) {
        if (!tokenInfo.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
            return ResultVo.businessError("帐号权限不足");
        }
        Company company = companyMapper.selectById(companyPo.getId());
        if (company == null) {
            return ResultVo.businessError("找不到相应企业信息");
        }
        if (companyPo.getExpiredDate() == null) {
            return ResultVo.businessError("有效期不能为空");
        }
        if (companyPo.getExpiredDate().getTime() < new Date().getTime()) {
            return ResultVo.businessError("有效期不能早于当前时间");
        }
        company.setExpiredDate(companyPo.getExpiredDate());

        companyMapper.updateById(company);
        return ResultVo.success();
    }

    @Override
    public ResultVo updateStatus(TokenInfo tokenInfo, CompanyPo companyPo) {
        if (!tokenInfo.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
            return ResultVo.businessError("帐号权限不足");
        }
        Company company = companyMapper.selectById(companyPo.getId());
        if (company == null) {
            return ResultVo.businessError("找不到相应企业信息");
        }
        company.setStatus(companyPo.getStatus());

        companyMapper.updateById(company);
        return ResultVo.success();
    }

    @Override
    public ResultVo updateCompany(TokenInfo tokenInfo, CompanyPo companyPo) {
        if (!tokenInfo.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
            return ResultVo.businessError("帐号权限不足");
        }
        Company company = companyMapper.selectById(companyPo.getId());
        if (company == null) {
            return ResultVo.businessError("找不到相应企业信息");
        }
        ValidateResult validate = companyPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }
        if (isCompanyNameExists(company.getId(), companyPo.getName())) {
            return ResultVo.businessError("企业名称已经存在");
        }
//        if (isCompanyAbbrExists(company.getId(), companyPo.getAbbr())) {
//            return ResultVo.businessError("企业简称已经存在");
//        }
        company.setName(companyPo.getName());
        company.setAddress(companyPo.getAddress());
        company.setContactName(companyPo.getContactName());
        company.setContactPhone(companyPo.getContactPhone());
        company.setProvince(companyPo.getProvince());
        company.setCity(companyPo.getCity());
        company.setDistrict(companyPo.getDistrict());
        company.setLevel(companyPo.getLevel());

        companyMapper.updateById(company);
        return ResultVo.success();
    }

    @Override
    public boolean isCompanyNameExists(Integer id, String companyName) {
        QueryWrapper<Company> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Company::getName, companyName);
        if (id != null) {
            queryWrapper.lambda().ne(Company::getId, id);
        }
        if (companyMapper.selectOne(queryWrapper) == null) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isCompanyAbbrExists(Integer id, String abbr) {
        QueryWrapper<Company> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Company::getAbbr, abbr);
        if (id != null) {
            queryWrapper.lambda().ne(Company::getId, id);
        }
        if (companyMapper.selectOne(queryWrapper) == null) {
            return false;
        }
        return true;
    }

    @Override
    public ResultVo listCompanys(TokenInfo tokenInfo, PageInfo<SearchCompanyPo> pageInfo) {
        pageInfo = PageInfo.checkParam(pageInfo);
        if (!tokenInfo.getType().equals(AccountTypeEnum.ADMIN.getKey())) {
            return ResultVo.businessError("帐号权限不足");
        }

        SearchCompanyPo searchPo = pageInfo.getRecords();

        QueryWrapper<Company> queryWrapper = new QueryWrapper<>();
        if (!StringUtils.isEmpty(searchPo.getName())) {
            queryWrapper.lambda().like(Company::getName, searchPo.getName());
        }

        Page<Company> page = new Page<>(pageInfo.getCurrent(), pageInfo.getSize());
        IPage<Company> selectPage = companyMapper.selectPage(page, queryWrapper);
        List<CompanyVo> companyVos = new ArrayList<>();
        if (selectPage.getRecords() != null) {
            selectPage.getRecords().forEach(company -> {
                CompanyVo companyVo = BeanUtil.transform(company, CompanyVo.class);
//                companyVo.setAccounts(accountService.listAccounts(companyVo.getId()));
                companyVo.setAccount(accountService.getMainAccount(tokenInfo, company.getId()));
                companyVos.add(companyVo);
            });
        }

        PageInfo<List<CompanyVo>> resultPageInfo = new PageInfo<>();
        resultPageInfo.setCurrent(pageInfo.getCurrent());
        resultPageInfo.setSize(pageInfo.getSize());
        resultPageInfo.setTotal(selectPage.getTotal());
        resultPageInfo.setRecords(companyVos);

        return ResultVo.success(resultPageInfo);
    }

    @Override
    public ResultVo myCompany(TokenInfo tokenInfo) {
        QueryWrapper<Company> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Company::getId, tokenInfo.getCompanyId());
        Company company = this.getOne(queryWrapper);
        CompanyVo companyVo = BeanUtil.transform(company, CompanyVo.class);
        return ResultVo.success(companyVo);
    }

    @Override
    public void checkExpiredCompanys() {
        QueryWrapper<Company> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Company::getStatus, 1);
        queryWrapper.lambda().lt(Company::getExpiredDate, DateUtil.format2Date(new Date(), DateUtil.YYYY_MM_DD));
        List<Company> companies = this.list(queryWrapper);
        if (companies == null || companies.isEmpty()) {
            log.error("没有超过有效期的公司");
            return;
        }
        log.error("{}个公司的有效期超过限制", companies.size());
        companies.forEach(company -> {
            company.setStatus(0);

            List<Account> accounts = this.accountService.listAccounts(null, company.getId(), null);
            if (accounts != null) {
                accounts.forEach(account -> {
                    account.setStatus(0);
                });
                this.accountService.updateBatchById(accounts);
            }
        });
        this.updateBatchById(companies);
    }
}
