package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.*;
import com.jair.entity.common.PageInfo;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.constant.*;
import com.jair.entity.po.*;
import com.jair.entity.vo.ContractVo;
import com.jair.entity.vo.DriverVo;
import com.jair.entity.vo.VehicleVo;
import com.jair.mapper.ContractMapper;
import com.jair.service.*;
import com.jair.util.BeanUtil;
import com.jair.util.DateUtil;
import com.jair.util.ObjectUtil;
import com.jair.util.StringAppend;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import static com.jair.entity.constant.ContractLogActionEnum.CREATE;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
@Slf4j
public class ContractServiceImpl extends ServiceImpl<ContractMapper, Contract> implements IContractService {

    @Autowired
    private IVehicleService vehicleService;
    @Autowired
    private ISourceService sourceService;
    @Autowired
    private IDriverService driverService;
    @Autowired
    private IContractOperationLogService logService;
    @Autowired
    private IAttFileService fileService;
    @Autowired
    private IContractSignAttachmentService contractAttachmentService;
    @Autowired
    private IContractReturnVehicleService returnVehicleService;
    @Autowired
    private IContractReturnVehicleAttachmentService returnVehicleAttachmentService;
    @Autowired
    private ISequenceService sequenceService;
    @Autowired
    private IPlanningService planningService;
    @Autowired
    private ContractMapper contractMapper;

    @Override
    public ResultVo listContracts(TokenInfo tokenInfo, PageInfo<SearchContractPo> pageInfo) {
        pageInfo = PageInfo.checkParam(pageInfo);
        List<ContractVo> contractVos = new ArrayList<>();
        SearchContractPo searchPo = pageInfo.getRecords();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            searchPo.setCompanyId(tokenInfo.getCompanyId());
        }

        Page<Contract> page = new Page<>(pageInfo.getCurrent(), pageInfo.getSize());
        IPage<Contract> selectPage = this.contractMapper.queryDriverContracts(page, searchPo);
        if (selectPage.getRecords() != null) {
            selectPage.getRecords().forEach(contract -> {
                Vehicle vehicle = vehicleService.getVehicleById(tokenInfo.getCompanyId(), contract.getVehicleId());
                VehicleVo vehicleVo = vehicleService.translateVehicleToVo(tokenInfo, vehicle);

                ContractVo contractVo = BeanUtil.transform(contract, ContractVo.class);
                contractVo.setVehicleVo(vehicleVo);

                if (contractVo.getSourceId() != null && contractVo.getSourceId() > 0) {
                    Source source = sourceService.getSourceById(tokenInfo.getCompanyId(), contractVo.getSourceId());
                    contractVo.setSourceVo(source);
                }
                if (contractVo.getDriverId() != null && contractVo.getDriverId() > 0) {
                    Driver driver = driverService.getDriverById(tokenInfo.getCompanyId(), contractVo.getDriverId());
                    DriverVo driverVo = driverService.translateDriverToVo(tokenInfo, driver);

                    contractVo.setDriverVo(driverVo);
                }

                contractVo.setFiles(contractAttachmentService.getFilesByContractId(contract.getId()));

                // 合同状态（是否有司机合同）
                boolean anotherContract = this.checkInEffectContract(contract.getVehicleId(), contract.getType() == 1 ? 2 : 1);
                contractVo.setAnotherContract(anotherContract == true ? 1 : 0);

                contractVos.add(contractVo);
            });
        }

        PageInfo<List<ContractVo>> resultPageInfo = new PageInfo<>();
        resultPageInfo.setCurrent(pageInfo.getCurrent());
        resultPageInfo.setSize(pageInfo.getSize());
        resultPageInfo.setTotal(selectPage.getTotal());
        resultPageInfo.setRecords(contractVos);

        return ResultVo.success(resultPageInfo);
    }

    @Override
    public ResultVo getContractLogs(TokenInfo tokenInfo, Integer contractId) {
        QueryWrapper<ContractOperationLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ContractOperationLog::getContractId, contractId);
        queryWrapper.lambda().orderByDesc(ContractOperationLog::getId);
        return ResultVo.success(logService.list(queryWrapper));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVo signContract(TokenInfo tokenInfo, SignContractPo signContractPo) throws Exception {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        ValidateResult validate = signContractPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }

        // 计算合同结束日期
        setContractEndDate(signContractPo);

        Vehicle vehicle = vehicleService.getVehicleById(tokenInfo.getCompanyId(), signContractPo.getVehicleId());
        if (vehicle == null) {
            return ResultVo.businessError("未找到相应车辆信息");
        }
        if (vehicle.getStatus() == 0) {
            return ResultVo.businessError("车辆状态不可用");
        }

        Contract contract = BeanUtil.transform(signContractPo, Contract.class);
        if (signContractPo.getType().equals(ContractTypeEnum.SOURCE_CONTRACT.getKey())) {
            Source source = sourceService.getSourceById(tokenInfo.getCompanyId(), signContractPo.getSourceId());
            if (source == null) {
                return ResultVo.businessError("未找到相应车源信息");
            }
        } else {
            Driver driver = driverService.getDriverById(tokenInfo.getCompanyId(), signContractPo.getDriverId());
            if (driver == null) {
                return ResultVo.businessError("未找到相应司机信息");
            }

            // 司机合同，校验合同结束日期
            Contract sourceContract = this.getContractByType(tokenInfo.getCompanyId(), 1, signContractPo.getVehicleId());
            if (sourceContract == null) {
                return ResultVo.businessError("未找到相应车源合同信息");
            }
            if (sourceContract.getStatus() == 0) {
                return ResultVo.businessError("车源合同状态不正确");
            }
            if (signContractPo.getEndDate().getTime() > sourceContract.getEndDate().getTime()) {
                return ResultVo.businessError(String.format("司机合同结束日期[%s]不能超过车源合同结束日期[%s]", DateUtil.format2Date(signContractPo.getEndDate()), DateUtil.format2Date(sourceContract.getEndDate())));
            }
            contract.setSourceId(sourceContract.getSourceId());
        }

        if (checkInEffectContract(vehicle.getId(), signContractPo.getType())) {
            return ResultVo.businessError("该车辆已经存在生效中的合同");
        }

        contract.setCreateTime(new Date());
        contract.setCode(genContractCode(signContractPo.getType()));
        contract.setCreateById(tokenInfo.getId());
        contract.setCreateByName(tokenInfo.getRealName());
        contract.setCompanyId(tokenInfo.getCompanyId());
        contract.setStatus(1); //有效

        // 设置押金状态
        if (contract.getPaidDepositAmount() < contract.getPayableDepositAmount()) {
            contract.setDepositStatus(DepositStatusEnum.ARREARAGE.getKey());
        } else {
            contract.setDepositStatus(DepositStatusEnum.NORMAL.getKey());
        }
        contract.setReturnDepositAmount(0);

        this.save(contract);

        // 押金不全设置，押金计划
        if (contract.getDepositStatus() == DepositStatusEnum.ARREARAGE.getKey()) {
            this.planningService.saveDepositPlanning(tokenInfo, contract);
        }

        // 生成租金计划
        this.planningService.saveRentPlanning(tokenInfo, contract, signContractPo.getFirstPaidAmount());

        // 生成费用项计划
        this.planningService.saveFeePlanning(tokenInfo, contract, signContractPo.getFees());

        // 保存合同附件
        contractAttachmentService.updateContractFiles(signContractPo.getFiles(), contract.getId());

        makeLog(tokenInfo, CREATE, null, contract);
        return ResultVo.success();
    }

    private void setContractEndDate(SignContractPo signContractPo) {
        Date startDate = signContractPo.getStartDate();
        Date endDate = null;
        if (signContractPo.getLeaseType() == LeaseTypeEnum.DAY_OF_MONTH.getKey()) {
            endDate = DateUtil.getAddTime(startDate, Calendar.DATE, signContractPo.getLeasePeriod() * 30);
        } else if (signContractPo.getLeaseType() == LeaseTypeEnum.MONTH.getKey()) {
            endDate = DateUtil.getAddTime(startDate, Calendar.MONTH, signContractPo.getLeasePeriod());
        } else if (signContractPo.getLeaseType() == LeaseTypeEnum.WEEK.getKey()) {
            endDate = DateUtil.getAddTime(startDate, Calendar.DATE, signContractPo.getLeasePeriod() * 7);
        }

        signContractPo.setEndDate(endDate);
    }

    @Override
    public ResultVo updateContract(TokenInfo tokenInfo, UpdateContractPo updateContractPo) {
        if (updateContractPo.getContractId() == null || updateContractPo.getContractId() == 0) {
            return ResultVo.businessError("未找到相应合同信息");
        }
        ValidateResult validate = updateContractPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }
        Contract originContract = getContractById(tokenInfo.getCompanyId(), updateContractPo.getContractId());
        if (originContract == null) {
            return ResultVo.businessError("未找到相应合同信息");
        }

        // 司机合同，校验合同结束日期
        if (originContract.getType().equals(ContractTypeEnum.DRIVER_CONTRACT.getKey())) {
            if (updateContractPo.getEndDate().getTime() > originContract.getEndDate().getTime()) {
                return ResultVo.businessError(String.format("司机合同结束日期[%s]不能超过车源合同结束日期[%s]", DateUtil.format2Date(updateContractPo.getEndDate()), DateUtil.format2Date(originContract.getEndDate())));
            }
        }

        Contract newContract = BeanUtil.transform(originContract, Contract.class);

        newContract.setStartDate(updateContractPo.getStartDate());
        newContract.setEndDate(updateContractPo.getEndDate());
        newContract.setOdograph(updateContractPo.getOdograph());
        newContract.setOdographEv(updateContractPo.getOdographEv());
        newContract.setLeaseType(updateContractPo.getLeaseType());
//        newContract.setDepositStatus(updateContractPo.getDepositStatus());
        newContract.setPayableDepositAmount(updateContractPo.getPayableDepositAmount());
        newContract.setPaidDepositAmount(updateContractPo.getPaidDepositAmount());
        if (newContract.getPaidDepositAmount() < newContract.getPayableDepositAmount()) {
            newContract.setDepositStatus(DepositStatusEnum.ARREARAGE.getKey());
        } else {
            newContract.setDepositStatus(DepositStatusEnum.NORMAL.getKey());
        }
//        newContract.setReturnDepositAmount(updateContractPo.getReturnDepositAmount());
        newContract.setRentAmount(updateContractPo.getRentAmount());
        newContract.setRemark(updateContractPo.getRemark());
        this.updateById(newContract);

        contractAttachmentService.updateContractFiles(updateContractPo.getFiles(), originContract.getId());

        makeLog(tokenInfo, ContractLogActionEnum.UPDATE, originContract, newContract);
        return ResultVo.success();
    }

    @Override
    public ResultVo returnDeposit(TokenInfo tokenInfo, ReturnDepositPo returnDepositPo) {
        if (returnDepositPo.getContractId() == null || returnDepositPo.getContractId() == 0) {
            return ResultVo.businessError("未找到相应合同信息");
        }
        Contract originContract = getContractById(tokenInfo.getCompanyId(), returnDepositPo.getContractId());
        if (originContract == null) {
            return ResultVo.businessError("未找到相应合同信息");
        }
        if (originContract.getStatus() == 1) {
            return ResultVo.businessError("合同正在执行中，不能退押金");
        }
        if (originContract.getDepositStatus() == DepositStatusEnum.ALL_RETURN.getKey()) {
            return ResultVo.businessError("押金已经全部返还");
        }
//        if (returnDepositPo.getReturnType() != 1 && returnDepositPo.getReturnType() != 2) {
//            return ResultVo.businessError("返还类型不正确");
//        }
        if (returnDepositPo.getReturnAmount() == null || returnDepositPo.getReturnAmount() <= 0) {
            return ResultVo.businessError("返还金额不正确");
        }
//        if (returnDepositPo.getReturnType() == 1 && originContract.getRentAmount() != returnDepositPo.getReturnAmount()) {
//            return ResultVo.businessError("返还类型和返还金额不匹配");
//        }

        Contract newContract = BeanUtil.transform(originContract, Contract.class);

        Integer returnAcount = newContract.getReturnDepositAmount() + returnDepositPo.getReturnAmount();
        newContract.setReturnDepositAmount(returnAcount);
        if (returnAcount >= newContract.getPaidDepositAmount()) {
            newContract.setDepositStatus(DepositStatusEnum.ALL_RETURN.getKey()); // 全部已退
        } else {
            newContract.setDepositStatus(DepositStatusEnum.PORTION_RETURN.getKey()); // 部分退还
        }
        this.updateById(newContract);
        makeLog(tokenInfo, ContractLogActionEnum.RETURN_DEPOSIT, originContract, newContract, returnDepositPo.getRemark());

        return ResultVo.success();
    }

    @Override
    public ResultVo terminateContract(TokenInfo tokenInfo, TerminateContractPo terminatePo) {
        if (terminatePo.getContractId() == null || terminatePo.getContractId() == 0) {
            return ResultVo.businessError("未找到相应合同信息");
        }
        Contract contract = getContractById(tokenInfo.getCompanyId(), terminatePo.getContractId());
        if (contract == null) {
            return ResultVo.businessError("未找到相应合同信息");
        }

        // 中止车源合同，先校验是否存在生效中的司机合同
        if (contract.getType().equals(ContractTypeEnum.SOURCE_CONTRACT.getKey())) {
            if (checkInEffectContract(contract.getVehicleId(), ContractTypeEnum.DRIVER_CONTRACT.getKey())) {
                return ResultVo.businessError("司机合同正在执行中，不能中止车源合同");
            }
        }

        contract.setStatus(0);
        contract.setDepositStatus(0);
        this.updateById(contract);

        if (terminatePo.getDeletePlanning() == 1) { //  删除所有收费计划
            this.planningService.disablePlanningByContractId(contract.getId());
        }

        makeLog(tokenInfo, ContractLogActionEnum.TERMINATE, null, contract);
        return ResultVo.success();
    }

    @Override
    public ResultVo returnVehicle(TokenInfo tokenInfo, ReturnVehiclePo returnVehiclePo) {
        if (returnVehiclePo.getContractId() == null || returnVehiclePo.getContractId() == 0) {
            return ResultVo.businessError("未找到相应合同信息");
        }
        Contract contract = getContractById(tokenInfo.getCompanyId(), returnVehiclePo.getContractId());
        if (contract == null) {
            return ResultVo.businessError("未找到相应合同信息");
        }
        if (!contract.getType().equals(ContractTypeEnum.DRIVER_CONTRACT.getKey())) {
            return ResultVo.businessError("合同类型错误");
        }
        if (contract.getStatus() == 1) {
            return ResultVo.businessError("合同正在执行中，不能上传交车信息");
        }

        ContractReturnVehicle contractReturnVehicle = BeanUtil.transform(returnVehiclePo, ContractReturnVehicle.class);
        contractReturnVehicle.setCreateTime(new Date());
        contractReturnVehicle.setCreateById(tokenInfo.getId());
        contractReturnVehicle.setCreateByName(tokenInfo.getRealName());
        contractReturnVehicle.setCompanyId(tokenInfo.getCompanyId());
        this.returnVehicleService.save(contractReturnVehicle);

        if (returnVehiclePo.getAccidentFiles() != null) {
            for (Integer fileId : returnVehiclePo.getAccidentFiles()) {
                ContractReturnVehicleAttachment contractSignAttachment = new ContractReturnVehicleAttachment();
                contractSignAttachment.setReturnId(contractReturnVehicle.getId());
                contractSignAttachment.setFileId(fileId);
                this.returnVehicleAttachmentService.save(contractSignAttachment);
            }
        }
        makeLog(tokenInfo, ContractLogActionEnum.RETURN_VEHICLE, null, contract);
        return ResultVo.success();
    }


    @Override
    public ResultVo getContractById(TokenInfo tokenInfo, Integer contractId) {
        Contract contract = this.getContractById(tokenInfo.getCompanyId(), contractId);
        ContractVo contractVo = BeanUtil.transform(contract, ContractVo.class);

        Vehicle vehicle = vehicleService.getVehicleById(tokenInfo.getCompanyId(), contract.getVehicleId());
        VehicleVo vehicleVo = vehicleService.translateVehicleToVo(tokenInfo, vehicle);

        contractVo.setVehicleVo(vehicleVo);

        if (contractVo.getSourceId() != null && contractVo.getSourceId() > 0) {
            Source source = sourceService.getSourceById(tokenInfo.getCompanyId(), contractVo.getSourceId());
            contractVo.setSourceVo(source);
        }
        if (contractVo.getDriverId() != null && contractVo.getDriverId() > 0) {
            Driver driver = driverService.getDriverById(tokenInfo.getCompanyId(), contractVo.getDriverId());
            DriverVo driverVo = driverService.translateDriverToVo(tokenInfo, driver);

            contractVo.setDriverVo(driverVo);
        }

        contractVo.setFiles(contractAttachmentService.getFilesByContractId(contract.getId()));
        return ResultVo.success(contractVo);
    }

    @Override
    public Contract getContractById(Integer companyId, Integer contractId) {
        QueryWrapper<Contract> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Contract::getCompanyId, companyId);
        queryWrapper.lambda().eq(Contract::getId, contractId);
        return this.getOne(queryWrapper);
    }

    @Override
    public Contract getContractByType(Integer companyId, Integer type, Integer id) {
        QueryWrapper<Contract> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Contract::getCompanyId, companyId);
        queryWrapper.lambda().eq(Contract::getType, type);
        queryWrapper.lambda().eq(Contract::getStatus, 1);
        if (type.equals(ContractTypeEnum.SOURCE_CONTRACT.getKey())) {
            queryWrapper.lambda().eq(Contract::getVehicleId, id);
        } else {
            queryWrapper.lambda().eq(Contract::getDriverId, id);
        }
        return this.getOne(queryWrapper);
    }

    @Override
    @Synchronized
    public String genContractCode(Integer type) {
        String pre = "EC";
        StringBuffer code = new StringBuffer();
        if (ContractTypeEnum.SOURCE_CONTRACT.getKey().equals(type)) {
            pre = "SC";
        } else if (ContractTypeEnum.DRIVER_CONTRACT.getKey().equals(type)) {
            pre = "DC";
        }

        code.append(pre);
        code.append(DateUtil.format2Date(new Date(), DateUtil.YYMMDD));
        code.append(String.format("%04d", sequenceService.getSequence() % 1000));
        code.append(getRandomNumber());

        return code.toString();
    }

    private String getRandomNumber() {
        int i = (int) (Math.random() * 1000);
        return String.format("%04d", i);
    }

    @Override
    public boolean checkInEffectContract(Integer vehicleId, Integer type) {
        QueryWrapper<Contract> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Contract::getVehicleId, vehicleId);
        if (type != null)
            queryWrapper.lambda().eq(Contract::getType, type);
        queryWrapper.lambda().eq(Contract::getStatus, 1);
        if (this.getOne(queryWrapper) == null) {
            return false;
        }
        return true;
    }

    private void makeLog(TokenInfo tokenInfo, ContractLogActionEnum action, Contract originContract, Contract newContract, String... extendsContent) {
        StringAppend content = new StringAppend(",");
        switch (action) {
            case CREATE: {
                content.append("创建合同");
                break;
            }
            case UPDATE: {
                if (!ObjectUtil.equals(originContract.getOdograph(), newContract.getOdograph())) {
                    content.append(String.format("里程从[%s]修改为[%s]", originContract.getOdograph(), newContract.getOdograph()));
                }
                if (!ObjectUtil.equals(originContract.getOdographEv(), newContract.getOdographEv())) {
                    content.append(String.format("EV里程从[%s]修改为[%s]", originContract.getOdographEv(), newContract.getOdographEv()));
                }
                if (!ObjectUtil.equals(originContract.getStartDate().getTime(), newContract.getStartDate().getTime())) {
                    content.append(String.format("开始日期从[%s]修改为[%s]", DateUtil.format2Date(originContract.getStartDate()), DateUtil.format2Date(newContract.getStartDate())));
                }
                if (!ObjectUtil.equals(originContract.getEndDate().getTime(), newContract.getEndDate().getTime())) {
                    content.append(String.format("结束日期从[%s]修改为[%s]", DateUtil.format2Date(originContract.getEndDate()), DateUtil.format2Date(newContract.getEndDate())));
                }
                if (!ObjectUtil.equals(originContract.getLeaseType(), newContract.getLeaseType())) {
                    content.append(String.format("租赁方式从[%s]修改为[%s]", LeaseTypeEnum.codeToName(originContract.getLeaseType()), LeaseTypeEnum.codeToName(newContract.getLeaseType())));
                }
                if (!ObjectUtil.equals(originContract.getDepositStatus(), newContract.getDepositStatus())) {
                    content.append(String.format("押金状态从[%s]修改为[%s]", DepositStatusEnum.codeToName(originContract.getDepositStatus()), DepositStatusEnum.codeToName(newContract.getDepositStatus())));
                }
                if (!ObjectUtil.equals(originContract.getPayableDepositAmount(), newContract.getPayableDepositAmount())) {
                    content.append(String.format("应付押金从[%s]修改为[%s]", originContract.getPayableDepositAmount(), newContract.getPayableDepositAmount()));
                }
                if (!ObjectUtil.equals(originContract.getPaidDepositAmount(), newContract.getPaidDepositAmount())) {
                    content.append(String.format("已付押金从[%s]修改为[%s]", originContract.getPaidDepositAmount(), newContract.getPaidDepositAmount()));
                }
                if (!ObjectUtil.equals(originContract.getReturnDepositAmount(), newContract.getReturnDepositAmount())) {
                    content.append(String.format("返还押金从[%s]修改为[%s]", originContract.getReturnDepositAmount(), newContract.getReturnDepositAmount()));
                }
                if (!ObjectUtil.equals(originContract.getRentAmount(), newContract.getRentAmount())) {
                    content.append(String.format("租金从[%s]修改为[%s]", originContract.getRentAmount(), newContract.getRentAmount()));
                }
                if (!ObjectUtil.equals(originContract.getRemark(), newContract.getRemark())) {
                    content.append(String.format("备注从[%s]修改为[%s]", originContract.getRemark(), newContract.getRemark()));
                }
                break;
            }
            case RENEW: {
                if (originContract.getStartDate().getTime() != newContract.getStartDate().getTime()) {
                    content.append(String.format("开始日期从[%s]修改为[%s]", DateUtil.format2Date(originContract.getStartDate()), DateUtil.format2Date(newContract.getEndDate())));
                }
                if (originContract.getEndDate().getTime() != newContract.getEndDate().getTime()) {
                    content.append(String.format("结束日期从[%s]修改为[%s]", DateUtil.format2Date(originContract.getEndDate()), DateUtil.format2Date(newContract.getEndDate())));
                }
                break;
            }
            case TERMINATE: {
                content.append("中止合同");
                break;
            }
            case RETURN_VEHICLE: {
                content.append("上传了交车信息");
                break;
            }
            case RETURN_DEPOSIT: {
                content.append(String.format("返还押金[%s]", newContract.getReturnDepositAmount() - originContract.getReturnDepositAmount()));
                if (newContract.getDepositStatus() == DepositStatusEnum.ALL_RETURN.getKey()) {
                    content.append("押金已全部返还");
                }
                break;
            }
        }

        if (extendsContent != null && extendsContent.length > 0) {
            StringAppend extendsString = new StringAppend(",");
            for (String s : extendsContent) {
                if (StringUtils.isEmpty(s)) {
                    continue;
                }
                extendsString.append(s);
            }
            if (!extendsString.isEmpty()) {
                content.append(extendsString);
            }
        }

        if (StringUtils.isEmpty(content.toString())) {
            return;
        }

        logService.createLog(tokenInfo, 1, newContract.getId(), action.getName(), content.toString());
    }

    @Override
    public void checkEndedContracts() {
        QueryWrapper<Contract> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().lt(Contract::getEndDate, new Date());
        queryWrapper.lambda().eq(Contract::getStatus, 1);
        queryWrapper.lambda().eq(Contract::getType, 2);

        List<Contract> contracts = this.list(queryWrapper);
        if (contracts == null || contracts.isEmpty()) {
            log.error("没有超过有效期的合同");
            return;
        }
        log.error("{}个合同的有效期超过限制", contracts.size());
        TokenInfo tokenInfo = new TokenInfo();
        tokenInfo.setId(0);
        tokenInfo.setRealName("系统");

        contracts.forEach(contract -> {
            TerminateContractPo terminatePo = new TerminateContractPo();
            terminatePo.setContractId(contract.getId());
            terminatePo.setDeletePlanning(0);
            this.terminateContract(tokenInfo, terminatePo);
        });
    }

    @Override
    public Integer getContractCountByCreateTime(TokenInfo tokenInfo, String createTime) {
        QueryWrapper<Contract> queryWrapper = new QueryWrapper<>();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            queryWrapper.lambda().eq(Contract::getCompanyId, tokenInfo.getCompanyId());
        }
        queryWrapper.lambda().likeRight(Contract::getCreateTime, createTime);
        queryWrapper.lambda().eq(Contract::getType, ContractTypeEnum.DRIVER_CONTRACT.getKey());
        return this.count(queryWrapper);
    }

    @Override
    public Integer getAllContractCount(TokenInfo tokenInfo) {
        QueryWrapper<Contract> queryWrapper = new QueryWrapper<>();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            queryWrapper.lambda().eq(Contract::getCompanyId, tokenInfo.getCompanyId());
        }
        queryWrapper.lambda().eq(Contract::getType, ContractTypeEnum.DRIVER_CONTRACT.getKey());
        return this.count(queryWrapper);
    }

    @Override
    public Integer getValidContractCount(TokenInfo tokenInfo) {
        QueryWrapper<Contract> queryWrapper = new QueryWrapper<>();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            queryWrapper.lambda().eq(Contract::getCompanyId, tokenInfo.getCompanyId());
        }
        queryWrapper.lambda().eq(Contract::getStatus, 1);
        queryWrapper.lambda().eq(Contract::getType, ContractTypeEnum.DRIVER_CONTRACT.getKey());
        return this.count(queryWrapper);
    }

    @Override
    public Integer getSignedDriverCount(TokenInfo tokenInfo) {
        QueryWrapper<Contract> queryWrapper = new QueryWrapper<>();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            queryWrapper.lambda().eq(Contract::getCompanyId, tokenInfo.getCompanyId());
        }
        queryWrapper.lambda().eq(Contract::getStatus, 1);
        queryWrapper.lambda().eq(Contract::getType, ContractTypeEnum.DRIVER_CONTRACT.getKey());
        queryWrapper.select("DISTINCT driver_id");
        return this.count(queryWrapper);
    }

    @Override
    public Integer getSignedVehicleCount(TokenInfo tokenInfo) {
        QueryWrapper<Contract> queryWrapper = new QueryWrapper<>();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            queryWrapper.lambda().eq(Contract::getCompanyId, tokenInfo.getCompanyId());
        }
        queryWrapper.lambda().eq(Contract::getStatus, 1);
        queryWrapper.lambda().eq(Contract::getType, ContractTypeEnum.DRIVER_CONTRACT.getKey());
        queryWrapper.select("DISTINCT vehicle_id");
        return this.count(queryWrapper);
    }
}
