package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.AttFile;
import com.jair.entity.bo.Company;
import com.jair.entity.bo.Vehicle;
import com.jair.entity.bo.VehicleLicense;
import com.jair.entity.common.PageInfo;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.constant.AccountTypeEnum;
import com.jair.entity.po.CreateVehiclePo;
import com.jair.entity.po.SearchVechiclePo;
import com.jair.entity.po.UpdateVehiclePo;
import com.jair.entity.po.ValidateResult;
import com.jair.entity.vo.CompanyVo;
import com.jair.entity.vo.VehicleVo;
import com.jair.mapper.VehicleLicenseMapper;
import com.jair.mapper.VehicleMapper;
import com.jair.service.*;
import com.jair.util.BeanUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 车辆信息 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
public class VehicleServiceImpl extends ServiceImpl<VehicleMapper, Vehicle> implements IVehicleService {

    @Autowired
    private IVehicleLicenseService vehicleLicenseService;
    @Autowired
    private IContractService sourceContractService;
    @Autowired
    private IAttFileService attFileService;
    @Autowired
    private IDataService dataService;
    @Autowired
    private VehicleMapper vehicleMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVo createVehicle(TokenInfo tokenInfo, CreateVehiclePo vehiclePo) {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        ValidateResult validate = vehiclePo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }
        if (isPlateNoExists(tokenInfo.getCompanyId(), null, vehiclePo.getPlateNo())) {
            return ResultVo.businessError("车牌号已经存在");
        }

        VehicleLicense license = BeanUtil.transform(vehiclePo, VehicleLicense.class);
        license.setCreateTime(new Date());
        license.setCreateById(tokenInfo.getId());
        license.setCreateByName(tokenInfo.getRealName());
        license.setCompanyId(tokenInfo.getCompanyId());
        vehicleLicenseService.save(license);
        if (license.getLicenseId() == null || license.getLicenseId() == 0) {
            return ResultVo.businessError("行驶证信息保存失败");
        }

        Vehicle vehicle = BeanUtil.transform(vehiclePo, Vehicle.class);
        vehicle.setCreateTime(new Date());
        vehicle.setCreateById(tokenInfo.getId());
        vehicle.setCreateByName(tokenInfo.getRealName());
        vehicle.setCompanyId(tokenInfo.getCompanyId());
        vehicle.setLicenseId(license.getLicenseId());
        vehicle.setStatus(1);
        this.save(vehicle);

        return ResultVo.success(vehicle);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVo updateVehicle(TokenInfo tokenInfo, UpdateVehiclePo vehiclePo) {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        ValidateResult validate = vehiclePo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }
        Vehicle vehicle = this.getVehicleById(tokenInfo.getCompanyId(), vehiclePo.getId());
        if (vehicle == null) {
            return ResultVo.businessError("车辆不存在");
        }
        if (isPlateNoExists(tokenInfo.getCompanyId(), vehiclePo.getId(), vehiclePo.getPlateNo())) {
            return ResultVo.businessError("车牌号已经存在");
        }

        BeanUtil.copyProperties(vehiclePo, vehicle);

        VehicleLicense license = vehicleLicenseService.getById(vehicle.getLicenseId());
        if (license != null) {
            BeanUtil.copyProperties(vehiclePo, license);
        }

        this.updateById(vehicle);
        this.vehicleLicenseService.updateById(license);

        return ResultVo.success();
    }

    @Override
    public ResultVo listVehicles(TokenInfo tokenInfo, PageInfo<SearchVechiclePo> pageInfo) {
        pageInfo = PageInfo.checkParam(pageInfo);
        SearchVechiclePo searchPo = pageInfo.getRecords();

        QueryWrapper<Vehicle> queryWrapper = new QueryWrapper<>();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            queryWrapper.lambda().like(Vehicle::getCompanyId, tokenInfo.getCompanyId());
        }
        if (!StringUtils.isEmpty(searchPo.getPlateNo())) {
            queryWrapper.lambda().like(Vehicle::getPlateNo, searchPo.getPlateNo());
        }
        if (!StringUtils.isEmpty(searchPo.getBrandName())) {
            queryWrapper.lambda().eq(Vehicle::getBrandName, searchPo.getBrandName());
        }

        Page<Vehicle> page = new Page<>(pageInfo.getCurrent(), pageInfo.getSize());
        IPage<Vehicle> selectPage = this.page(page, queryWrapper);

        List<VehicleVo> vehicleVos = new ArrayList<>();
        if (selectPage.getRecords() != null) {
            selectPage.getRecords().forEach(vehicle -> {
                VehicleVo vehicleVo = translateVehicleToVo(tokenInfo, vehicle);
                vehicleVos.add(vehicleVo);
                // TODO 获取 合同状态
            });
        }

        PageInfo<List<VehicleVo>> resultPageInfo = new PageInfo<>();
        resultPageInfo.setCurrent(pageInfo.getCurrent());
        resultPageInfo.setSize(pageInfo.getSize());
        resultPageInfo.setTotal(selectPage.getTotal());
        resultPageInfo.setRecords(vehicleVos);

        return ResultVo.success(resultPageInfo);
    }

    @Override
    public ResultVo listNoSourceContractVehicles(TokenInfo tokenInfo) {
        return ResultVo.success(vehicleMapper.listNoSourceContractVehicles(tokenInfo.getCompanyId()));
    }

    @Override
    public ResultVo listAllowRentVehicles(TokenInfo tokenInfo) {
        return ResultVo.success(vehicleMapper.listAllowRentVehicles(tokenInfo.getCompanyId()));
    }

    @Override
    public ResultVo returnVehicle(TokenInfo tokenInfo, Integer vehicleId) {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }

        Vehicle vehicle = getVehicleById(tokenInfo.getCompanyId(), vehicleId);
        if (vehicle == null) {
            return ResultVo.businessError("未找到该车辆");
        }
        if (sourceContractService.checkInEffectContract(vehicle.getId(), null)) {
            return ResultVo.businessError("该车辆存在生效的合同，不能进行退车操作");
        }

        // TODO 检查司机合同

        vehicle.setStatus(0);
        this.updateById(vehicle);

        return ResultVo.success();
    }


    @Override
    public boolean isPlateNoExists(Integer companyId, Integer vehicleId, String plateNo) {
        QueryWrapper<Vehicle> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Vehicle::getCompanyId, companyId);
        queryWrapper.lambda().eq(Vehicle::getPlateNo, plateNo);
        if (vehicleId != null) {
            queryWrapper.lambda().ne(Vehicle::getId, vehicleId);
        }
        if (this.getOne(queryWrapper) == null) {
            return false;
        }
        return true;
    }

    @Override
    public ResultVo getVehicleById(TokenInfo tokenInfo, Integer vehicleId) {
        Vehicle vehicle = this.getVehicleById(tokenInfo.getCompanyId(), vehicleId);
        VehicleVo vehicleVo = this.translateVehicleToVo(tokenInfo, vehicle);
        return ResultVo.success(vehicleVo);
    }

    @Override
    public Vehicle getVehicleById(Integer companyId, Integer vehicleId) {
        QueryWrapper<Vehicle> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Vehicle::getId, vehicleId);
        queryWrapper.lambda().eq(Vehicle::getCompanyId, companyId);
        return this.getOne(queryWrapper);
    }

    @Override
    public VehicleVo translateVehicleToVo(TokenInfo tokenInfo, Vehicle vehicle) {
        VehicleVo vehicleVo = BeanUtil.transform(vehicle, VehicleVo.class);

        VehicleLicense license = this.vehicleLicenseService.getById(vehicle.getLicenseId());
        if (license != null) {
            BeanUtil.copyProperties(license, vehicleVo);

            vehicleVo.setBrandId(dataService.getBrandIdByName(tokenInfo, vehicle.getBrandName()));

            if (license.getMainFileId() != null) {
                AttFile attFile = attFileService.getAttFileById(vehicle.getCompanyId(), license.getMainFileId());
                if (attFile != null) {
                    vehicleVo.setMainFileUrl(attFile.getFilePath());
                }
            }
            if (license.getSecondFileId() != null) {
                AttFile attFile = attFileService.getAttFileById(vehicle.getCompanyId(), license.getSecondFileId());
                if (attFile != null) {
                    vehicleVo.setSecondFileUrl(attFile.getFilePath());
                }
            }
        }
        return vehicleVo;
    }

    @Override
    public Integer getAllVehicleCount(TokenInfo tokenInfo) {
        QueryWrapper<Vehicle> queryWrapper = new QueryWrapper<>();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            queryWrapper.lambda().eq(Vehicle::getCompanyId, tokenInfo.getCompanyId());
        }
        return this.count(queryWrapper);
    }
}
